import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { AggregateID, UniqueConstraintException } from '@mobicoop/ddd-library';
import { USER_REPOSITORY } from '@modules/user/user.di-tokens';
import { UserRepositoryPort } from '../../ports/user.repository.port';
import { UserEntity } from '../../../domain/user.entity';
import { UpdateUserCommand } from './update-user.command';
import {
  EmailAlreadyExistsException,
  PhoneAlreadyExistsException,
} from '@modules/user/core/domain/user.errors';

@CommandHandler(UpdateUserCommand)
export class UpdateUserService implements ICommandHandler {
  constructor(
    @Inject(USER_REPOSITORY)
    private readonly userRepository: UserRepositoryPort,
  ) {}

  async execute(command: UpdateUserCommand): Promise<AggregateID> {
    try {
      const user: UserEntity = await this.userRepository.findOneById(
        command.id,
      );
      user.update({
        firstName: command.firstName,
        lastName: command.lastName,
        email: command.email,
        phone: command.phone,
      });
      await this.userRepository.update(user.id, user);
      return user.id;
    } catch (error: any) {
      if (
        error instanceof UniqueConstraintException &&
        error.message.includes('email')
      ) {
        throw new EmailAlreadyExistsException(error);
      }
      if (
        error instanceof UniqueConstraintException &&
        error.message.includes('phone')
      ) {
        throw new PhoneAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
