import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { UserUpdatedDomainEvent } from '../../domain/events/user-updated.domain-event';
import { UserUpdatedIntegrationEvent } from '../events/user-updated.integration-event';
import { USER_UPDATED_ROUTING_KEY } from '@modules/user/user.constants';

@Injectable()
export class PublishMessageWhenUserIsUpdatedDomainEventHandler {
  constructor(
    @Inject(USER_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(UserUpdatedDomainEvent.name, { async: true, promisify: true })
  async handle(event: UserUpdatedDomainEvent): Promise<any> {
    const userUpdatedIntegrationEvent = new UserUpdatedIntegrationEvent({
      id: event.aggregateId,
      firstName: event.firstName,
      lastName: event.lastName,
      email: event.email,
      phone: event.phone,
      metadata: event.metadata,
    });
    this.messagePublisher.publish(
      USER_UPDATED_ROUTING_KEY,
      JSON.stringify(userUpdatedIntegrationEvent),
    );
  }
}
