import { Test, TestingModule } from '@nestjs/testing';
import { PublishMessageWhenUserIsCreatedDomainEventHandler } from '@modules/user/core/application/event-handlers/publish-message-when-user-is-created.domain-event-handler';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { UserCreatedDomainEvent } from '@modules/user/core/domain/events/user-created.domain-event';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when user is created domain event handler', () => {
  let publishMessageWhenUserIsCreatedDomainEventHandler: PublishMessageWhenUserIsCreatedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenUserIsCreatedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenUserIsCreatedDomainEventHandler =
      module.get<PublishMessageWhenUserIsCreatedDomainEventHandler>(
        PublishMessageWhenUserIsCreatedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const userCreatedDomainEvent: UserCreatedDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      firstName: 'John',
      lastName: 'Doe',
      email: 'john.doe@email.com',
      phone: '+33611223344',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenUserIsCreatedDomainEventHandler.handle(
      userCreatedDomainEvent,
    );
    expect(publishMessageWhenUserIsCreatedDomainEventHandler).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      'user.created',
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000},"firstName":"John","lastName":"Doe","email":"john.doe@email.com","phone":"+33611223344"}',
    );
  });
});
